@@ -138,8 +138,8 @@ impl<'a> Compiler<'a> {
         };
 
         let mut num_cards = 0usize;
-        for n in compilation_unit.iter() {
-            self.current_index.lane = n.name.to_string();
+        for (i, n) in compilation_unit.iter().enumerate() {
+            self.current_index.lane = i;
             self.current_index.card_index.indices.clear();
             num_cards += n.cards.len();
 
@@ -174,11 +174,11 @@ impl<'a> Compiler<'a> {
                 Ok(i) => i,
                 Err(_) => return Err(self.error(CompilationErrorPayload::TooManyCards(il))),
             };
-            self.current_index = CardIndex::new(&main_lane.name, 0);
+            self.current_index = CardIndex::new(il, 0);
             self.scope_begin();
             self.process_lane(il, main_lane)?;
             self.current_index = CardIndex {
-                lane: main_lane.name.to_string(),
+                lane: il,
                 card_index: LaneCardIndex {
                     indices: smallvec::smallvec![len],
                 },
@@ -190,7 +190,7 @@ impl<'a> Compiler<'a> {
 
         for (il, lane) in lanes {
             self.current_index = CardIndex {
-                lane: lane.name.to_string(),
+                lane: il,
                 ..Default::default()
             };
             let nodeid_hash = self.current_index.as_handle();
@@ -479,7 +479,8 @@ impl<'a> Compiler<'a> {
                 })?;
                 self.scope_end();
             }
-            Card::While { condition, body } => {
+            Card::While(children) => {
+                let [condition, body] = &**children;
                 let block_begin = self.program.bytecode.len() as i32;
                 self.current_index.push_subindex(0);
                 self.process_card(condition)?;
@@ -508,7 +509,7 @@ impl<'a> Compiler<'a> {
                 };
                 self.write_local_var(loop_n_index);
                 // init counter to 0
-                self.process_card(&Card::ScalarInt(IntegerNode(0)))?;
+                self.process_card(&Card::ScalarInt(0))?;
                 self.write_local_var(loop_counter_index);
 
                 let block_begin = self.program.bytecode.len() as i32;
@@ -526,7 +527,7 @@ impl<'a> Compiler<'a> {
                     c.process_card(repeat)?;
                     c.current_index.pop_subindex();
                     // i = i + 1
-                    c.process_card(&Card::ScalarInt(IntegerNode(1)))?;
+                    c.process_card(&Card::ScalarInt(1))?;
                     c.read_local_var(loop_counter_index);
                     c.process_card(&Card::Add)?;
                     c.write_local_var(loop_counter_index);
@@ -572,10 +573,9 @@ impl<'a> Compiler<'a> {
                     .or_insert_with(move || *variable.0);
                 write_to_vec(*id, &mut self.program.bytecode);
             }
-            Card::IfElse {
-                then: then_card,
-                r#else: else_card,
-            } => {
+            Card::IfElse(children) => {
+                let [then_card, else_card] = &**children;
+
                 let mut idx = 0;
                 self.current_index.push_subindex(0);
                 self.encode_if_then(Instruction::GotoIfFalse, |c| {
@@ -613,10 +613,10 @@ impl<'a> Compiler<'a> {
                 write_to_vec(key, &mut self.program.bytecode);
             }
             Card::ScalarInt(s) => {
-                write_to_vec(s.0, &mut self.program.bytecode);
+                write_to_vec(*s, &mut self.program.bytecode);
             }
             Card::ScalarFloat(s) => {
-                write_to_vec(s.0, &mut self.program.bytecode);
+                write_to_vec(*s, &mut self.program.bytecode);
             }
             Card::ScalarNil
             | Card::Return
