@@ -36,16 +36,14 @@ pub enum Card {
     Len,
     SetProperty,
     GetProperty,
-    ScalarInt(IntegerNode),
-    ScalarFloat(FloatNode),
+    ScalarInt(i64),
+    ScalarFloat(f64),
     StringLiteral(StringNode),
     CallNative(Box<CallNode>),
     IfTrue(Box<Card>),
     IfFalse(Box<Card>),
-    IfElse {
-        then: Box<Card>,
-        r#else: Box<Card>,
-    },
+    /// Children = [then, else]
+    IfElse(Box<[Card; 2]>),
     Jump(LaneNode),
     SetGlobalVar(VarNode),
     SetVar(VarNode),
@@ -55,10 +53,9 @@ pub enum Card {
         i: Option<VarNode>,
         body: Box<Card>,
     },
-    While {
-        condition: Box<Card>,
-        body: Box<Card>,
-    },
+    /// Children = [condition, body]
+    While(Box<[Card; 2]>),
+    // TODO: move the entire variant into a struct
     ForEach {
         /// Loop variable is written into this variable
         i: Option<VarNode>,
@@ -209,40 +206,6 @@ impl Card {
         };
     }
 
-    /// If the card has sub-cards then return references to them in a list
-    pub fn as_card_list(&self) -> Option<smallvec::SmallVec<[&Card; 4]>> {
-        let mut res = smallvec::smallvec![];
-        match self {
-            Self::CompositeCard(c) => {
-                res.extend(c.cards.iter());
-            }
-            Self::IfTrue(c) | Self::IfFalse(c) => res.push(c.as_ref()),
-            Self::IfElse { then, r#else } => {
-                res.push(then.as_ref());
-                res.push(r#else.as_ref());
-            }
-            _ => return None,
-        }
-        Some(res)
-    }
-
-    /// If the card has sub-cards then return references to them in a list
-    pub fn as_card_list_mut(&mut self) -> Option<smallvec::SmallVec<[&mut Card; 4]>> {
-        let mut res = smallvec::smallvec![];
-        match self {
-            Self::CompositeCard(c) => {
-                res.extend(c.cards.iter_mut());
-            }
-            Self::IfTrue(c) | Self::IfFalse(c) => res.push(c.as_mut()),
-            Self::IfElse { then, r#else } => {
-                res.push(then.as_mut());
-                res.push(r#else.as_mut());
-            }
-            _ => return None,
-        }
-        Some(res)
-    }
-
     pub fn as_composite_card(&self) -> Option<&CompositeCard> {
         if let Self::CompositeCard(v) = self {
             Some(v)
@@ -282,7 +245,7 @@ impl Card {
     }
 
     pub fn scalar_int(i: i64) -> Self {
-        Card::ScalarInt(IntegerNode(i))
+        Card::ScalarInt(i)
     }
 
     pub fn string_card(s: impl Into<String>) -> Self {
@@ -293,47 +256,61 @@ impl Card {
         Self::Jump(LaneNode(s.into()))
     }
 
-    pub fn get_card_by_index_mut(&mut self, i: usize) -> Option<&mut Card> {
+    pub fn get_child_mut(&mut self, i: usize) -> Option<&mut Card> {
         let res;
         match self {
             Card::CompositeCard(c) => res = c.cards.get_mut(i)?,
-            Card::IfTrue(c) | Card::IfFalse(c) => {
+            Card::Repeat { i: _, body: c } | Card::IfTrue(c) | Card::IfFalse(c) => {
                 if i != 0 {
                     return None;
                 }
                 res = c;
             }
-            Card::IfElse { then, r#else } => {
+            Card::ForEach {
+                i: _,
+                k: _,
+                v: _,
+                variable: a,
+                body: b,
+            } => {
                 if i > 1 {}
                 match i {
-                    0 => res = then.as_mut(),
-                    1 => res = r#else.as_mut(),
+                    0 => res = a.as_mut(),
+                    1 => res = b.as_mut(),
                     _ => return None,
                 }
             }
+            Card::While(children) | Card::IfElse(children) => return children.get_mut(i),
             _ => return None,
         }
         Some(res)
     }
 
-    pub fn get_card_by_index(&self, i: usize) -> Option<&Card> {
+    pub fn get_child(&self, i: usize) -> Option<&Card> {
         let res;
         match self {
             Card::CompositeCard(c) => res = c.cards.get(i)?,
-            Card::IfTrue(c) | Card::IfFalse(c) => {
+            Card::Repeat { i: _, body: c } | Card::IfTrue(c) | Card::IfFalse(c) => {
                 if i != 0 {
                     return None;
                 }
                 res = c;
             }
-            Card::IfElse { then, r#else } => {
+            Card::ForEach {
+                i: _,
+                k: _,
+                v: _,
+                variable: a,
+                body: b,
+            } => {
                 if i > 1 {}
                 match i {
-                    0 => res = then.as_ref(),
-                    1 => res = r#else.as_ref(),
+                    0 => res = a.as_ref(),
+                    1 => res = b.as_ref(),
                     _ => return None,
                 }
             }
+            Card::While(children) | Card::IfElse(children) => return children.get(i),
             _ => return None,
         }
         Some(res)
@@ -348,20 +325,33 @@ impl Card {
                 }
                 res = c.cards.remove(i);
             }
-            Card::IfTrue(c) | Card::IfFalse(c) => {
+            Card::Repeat { i: _, body: c } | Card::IfTrue(c) | Card::IfFalse(c) => {
                 if i != 0 {
                     return None;
                 }
                 res = std::mem::replace::<Card>(c.as_mut(), Card::Pass);
             }
-            Card::IfElse { then, r#else } => {
+
+            Card::ForEach {
+                i: _,
+                k: _,
+                v: _,
+                variable: a,
+                body: b,
+            } => {
                 if i > 1 {}
                 match i {
-                    0 => res = std::mem::replace::<Card>(then.as_mut(), Card::Pass),
-                    1 => res = std::mem::replace::<Card>(r#else.as_mut(), Card::Pass),
+                    0 => res = std::mem::replace::<Card>(a.as_mut(), Card::Pass),
+                    1 => res = std::mem::replace::<Card>(b.as_mut(), Card::Pass),
                     _ => return None,
                 }
             }
+            Card::While(children) | Card::IfElse(children) => {
+                let Some(c) = children.get_mut(i) else {
+                    return None
+                };
+                res = std::mem::replace(c, Card::Pass);
+            }
             _ => return None,
         }
         Some(res)
@@ -379,20 +369,32 @@ impl Card {
                 }
                 c.cards.insert(i, card);
             }
-            Card::IfTrue(c) | Card::IfFalse(c) => {
+            Card::Repeat { i: _, body: c } | Card::IfTrue(c) | Card::IfFalse(c) => {
                 if i != 0 {
                     return Err(card);
                 }
                 *c.as_mut() = card;
             }
-            Card::IfElse { then, r#else } => {
+
+            Card::ForEach {
+                i: _,
+                k: _,
+                v: _,
+                variable: a,
+                body: b,
+            } => {
                 if i > 1 {}
                 match i {
-                    0 => *then.as_mut() = card,
-                    1 => *r#else.as_mut() = card,
+                    0 => *a.as_mut() = card,
+                    1 => *b.as_mut() = card,
                     _ => return Err(card),
                 };
             }
+            Card::While(children) | Card::IfElse(children) => {
+                if let Some(c) = children.get_mut(i) {
+                    *c = card;
+                }
+            }
             _ => return Err(card),
         }
         Ok(())
@@ -405,31 +407,35 @@ impl Card {
                 Some(c) => std::mem::replace(c, card),
                 None => return Err(card),
             },
-            Card::IfTrue(c) | Card::IfFalse(c) => {
+            Card::Repeat { i: _, body: c } | Card::IfTrue(c) | Card::IfFalse(c) => {
                 if i != 0 {
                     return Err(card);
                 }
                 std::mem::replace(c.as_mut(), card)
             }
-            Card::IfElse { then, r#else } => match i {
-                0 => std::mem::replace(then.as_mut(), card),
-                1 => std::mem::replace(r#else.as_mut(), card),
+            Card::ForEach {
+                i: _,
+                k: _,
+                v: _,
+                variable: a,
+                body: b,
+            } => match i {
+                0 => std::mem::replace(a.as_mut(), card),
+                1 => std::mem::replace(b.as_mut(), card),
                 _ => return Err(card),
             },
+            Card::While(children) | Card::IfElse(children) => {
+                let Some(c) = children.get_mut(i) else {
+                    return Err(card);
+                };
+                std::mem::replace(c, card)
+            }
             _ => return Err(card),
         };
         Ok(res)
     }
 }
 
-#[derive(Debug, Clone, Default, Copy)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-pub struct IntegerNode(pub i64);
-
-#[derive(Debug, Clone, Default, Copy)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-pub struct FloatNode(pub f64);
-
 #[derive(Debug, Clone, Default)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct CallNode(pub InputString);
