@@ -72,10 +72,7 @@ pub fn get_instruction_descriptions() -> Vec<SubProgramDescription> {
         get_desc(&Card::StringLiteral(Default::default())),
         get_desc(&Card::IfTrue(Default::default())),
         get_desc(&Card::IfFalse(Default::default())),
-        get_desc(&Card::IfElse {
-            then: Default::default(),
-            r#else: Default::default(),
-        }),
+        get_desc(&Card::IfElse(Default::default())),
         get_desc(&Card::Jump(Default::default())),
         get_desc(&Card::SetGlobalVar(Default::default())),
         get_desc(&Card::SetVar(Default::default())),
@@ -84,10 +81,7 @@ pub fn get_instruction_descriptions() -> Vec<SubProgramDescription> {
             i: None,
             body: Default::default(),
         }),
-        get_desc(&Card::While {
-            condition: Default::default(),
-            body: Default::default(),
-        }),
+        get_desc(&Card::While(Default::default())),
         get_desc(&Card::ForEach {
             i: None,
             k: None,
@@ -98,7 +92,7 @@ pub fn get_instruction_descriptions() -> Vec<SubProgramDescription> {
     ]
 }
 
-pub fn get_desc(node: &Card) -> SubProgramDescription {
+fn get_desc(node: &Card) -> SubProgramDescription {
     match node {
         Card::CompositeCard { .. } | Card::CallNative(_) => unreachable!(),
         Card::GetProperty => subprogram_description!(
