@@ -8,7 +8,6 @@ use crate::compiler::Lane;
 use crate::prelude::CompilationErrorPayload;
 use smallvec::SmallVec;
 use std::collections::hash_map::DefaultHasher;
-use std::collections::BTreeMap;
 use std::hash::Hasher;
 use std::rc::Rc;
 use thiserror::Error;
@@ -27,8 +26,8 @@ pub enum IntoStreamError {
 pub type CaoProgram = Module;
 pub type CaoIdentifier = String;
 pub type Imports = Vec<CaoIdentifier>;
-pub type Lanes = BTreeMap<CaoIdentifier, Lane>;
-pub type Submodules = BTreeMap<CaoIdentifier, Module>;
+pub type Lanes = Vec<(CaoIdentifier, Lane)>;
+pub type Submodules = Vec<(CaoIdentifier, Module)>;
 
 #[derive(Debug, Clone, Default)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
@@ -45,14 +44,14 @@ pub struct Module {
 #[derive(Default, Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct CardIndex {
-    pub lane: String,
+    pub lane: usize,
     pub card_index: LaneCardIndex,
 }
 
 impl CardIndex {
-    pub fn new(lane: &str, card_index: usize) -> Self {
+    pub fn new(lane: usize, card_index: usize) -> Self {
         Self {
-            lane: lane.to_owned(),
+            lane,
             card_index: LaneCardIndex::new(card_index),
         }
     }
@@ -66,7 +65,7 @@ impl CardIndex {
     }
 
     pub fn as_handle(&self) -> crate::prelude::Handle {
-        let lane_handle = crate::prelude::Handle::from_bytes(self.lane.as_bytes());
+        let lane_handle = crate::prelude::Handle::from_u64(self.lane as u64);
         let subindices = self.card_index.indices.as_slice();
         let sub_handle = unsafe {
             crate::prelude::Handle::from_bytes(std::slice::from_raw_parts(
@@ -175,9 +174,9 @@ pub enum CardFetchError {
 
 impl Module {
     pub fn get_card_mut<'a>(&'a mut self, idx: &CardIndex) -> Result<&'a mut Card, CardFetchError> {
-        let lane = self
+        let (_, lane) = self
             .lanes
-            .get_mut(idx.lane.as_str())
+            .get_mut(idx.lane)
             .ok_or(CardFetchError::LaneNotFound)?;
         let mut card = lane
             .cards
@@ -186,7 +185,7 @@ impl Module {
 
         for i in &idx.card_index.indices[1..] {
             card = card
-                .get_card_by_index_mut(*i as usize)
+                .get_child_mut(*i as usize)
                 .ok_or(CardFetchError::CardNotFound { depth: *i as usize })?;
         }
 
@@ -194,9 +193,9 @@ impl Module {
     }
 
     pub fn get_card<'a>(&'a self, idx: &CardIndex) -> Result<&'a Card, CardFetchError> {
-        let lane = self
+        let (_, lane) = self
             .lanes
-            .get(idx.lane.as_str())
+            .get(idx.lane)
             .ok_or(CardFetchError::LaneNotFound)?;
         let mut card = lane
             .cards
@@ -205,7 +204,7 @@ impl Module {
 
         for i in &idx.card_index.indices[1..] {
             card = card
-                .get_card_by_index(*i as usize)
+                .get_child(*i as usize)
                 .ok_or(CardFetchError::CardNotFound { depth: *i as usize })?;
         }
 
@@ -213,9 +212,9 @@ impl Module {
     }
 
     pub fn remove_card(&mut self, idx: &CardIndex) -> Result<Card, CardFetchError> {
-        let lane = self
+        let (_, lane) = self
             .lanes
-            .get_mut(idx.lane.as_str())
+            .get_mut(idx.lane)
             .ok_or(CardFetchError::LaneNotFound)?;
         if idx.card_index.indices.len() == 1 {
             if lane.cards.len() <= idx.card_index.indices[0] as usize {
@@ -232,7 +231,7 @@ impl Module {
         let len = idx.card_index.indices.len();
         for i in &idx.card_index.indices[1..(len - 1).max(1)] {
             card = card
-                .get_card_by_index_mut(*i as usize)
+                .get_child_mut(*i as usize)
                 .ok_or(CardFetchError::CardNotFound { depth: *i as usize })?;
         }
         let i = *idx.card_index.indices.last().unwrap() as usize;
@@ -242,9 +241,9 @@ impl Module {
 
     /// Return the old card
     pub fn replace_card(&mut self, idx: &CardIndex, child: Card) -> Result<Card, CardFetchError> {
-        let lane = self
+        let (_, lane) = self
             .lanes
-            .get_mut(idx.lane.as_str())
+            .get_mut(idx.lane)
             .ok_or(CardFetchError::LaneNotFound)?;
         if idx.card_index.indices.len() == 1 {
             let c = lane
@@ -263,7 +262,7 @@ impl Module {
         let len = idx.card_index.indices.len();
         for i in &idx.card_index.indices[1..(len - 1).max(1)] {
             card = card
-                .get_card_by_index_mut(*i as usize)
+                .get_child_mut(*i as usize)
                 .ok_or(CardFetchError::CardNotFound { depth: *i as usize })?;
         }
         let i = *idx.card_index.indices.last().unwrap() as usize;
@@ -272,9 +271,9 @@ impl Module {
     }
 
     pub fn insert_card(&mut self, idx: &CardIndex, child: Card) -> Result<(), CardFetchError> {
-        let lane = self
+        let (_, lane) = self
             .lanes
-            .get_mut(idx.lane.as_str())
+            .get_mut(idx.lane)
             .ok_or(CardFetchError::LaneNotFound)?;
         if idx.card_index.indices.len() == 1 {
             if lane.cards.len() < idx.card_index.indices[0] as usize {
@@ -292,7 +291,7 @@ impl Module {
         let len = idx.card_index.indices.len();
         for i in &idx.card_index.indices[1..(len - 1).max(1)] {
             card = card
-                .get_card_by_index_mut(*i as usize)
+                .get_child_mut(*i as usize)
                 .ok_or(CardFetchError::CardNotFound { depth: *i as usize })?;
         }
         let i = *idx.card_index.indices.last().unwrap() as usize;
@@ -305,13 +304,18 @@ impl Module {
         mut self,
         recursion_limit: u32,
     ) -> Result<Vec<LaneIr>, CompilationErrorPayload> {
+        self.ensure_invariants(&mut Default::default())?;
         // the first lane is special
         //
-        let first_fn = self
+        let (main_index, _) = self
             .lanes
-            .remove("main")
+            .iter()
+            .enumerate()
+            .find(|(_, (name, _))| name == "main")
             .ok_or(CompilationErrorPayload::NoMain)?;
 
+        let (_, first_fn) = self.lanes.swap_remove(main_index);
+
         let imports = self.execute_imports()?;
         let first_fn = lane_to_lane_ir(&first_fn, &["main"], Rc::new(imports));
         let mut result = vec![first_fn];
@@ -325,6 +329,24 @@ impl Module {
         Ok(result)
     }
 
+    fn ensure_invariants<'a>(
+        &'a self,
+        aux: &mut std::collections::HashSet<&'a str>,
+    ) -> Result<(), CompilationErrorPayload> {
+        // test that submodule names are unique
+        for (name, _) in self.submodules.iter() {
+            if aux.contains(name.as_str()) {
+                return Err(CompilationErrorPayload::DuplicateModule(name.to_string()));
+            }
+            aux.insert(name.as_str());
+        }
+        for (_, module) in self.submodules.iter() {
+            aux.clear();
+            module.ensure_invariants(aux)?;
+        }
+        Ok(())
+    }
+
     fn execute_imports(&self) -> Result<ImportsIr, CompilationErrorPayload> {
         let mut result = ImportsIr::with_capacity(self.imports.len());
 
