@@ -1,11 +1,8 @@
-use std::collections::BTreeMap;
-
 use super::*;
 
 #[test]
 fn composite_card_test() {
-    let mut lanes = BTreeMap::new();
-    lanes.insert(
+    let lanes = vec![(
         "main".into(),
         Lane::default().with_card(Card::CompositeCard(Box::new(CompositeCard {
             ty: "triplepog".to_string(),
@@ -15,7 +12,7 @@ fn composite_card_test() {
                 Card::StringLiteral(StringNode("poggers".to_owned())),
             ],
         }))),
-    );
+    )];
     let cu = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
@@ -70,23 +67,52 @@ fn empty_varname_is_error() {
 
 #[test]
 fn can_call_nested_function_test() {
-    let mut submodules = BTreeMap::new();
-    submodules.insert(
+    let submodules = vec![(
         "coggers".into(),
         Module {
             imports: Default::default(),
             submodules: Default::default(),
-            lanes: BTreeMap::from([("pooh".into(), Lane::default().with_card(Card::Pass))]),
+            lanes: vec![("pooh".into(), Lane::default().with_card(Card::Pass))],
         },
-    );
+    )];
     let prog = CaoProgram {
         imports: Default::default(),
         submodules,
-        lanes: BTreeMap::from([(
+        lanes: vec![(
             "main".into(),
             Lane::default().with_cards(vec![Card::Jump(LaneNode("coggers.pooh".to_string()))]),
-        )]),
+        )],
     };
 
     compile(prog, None).unwrap();
 }
+
+#[test]
+fn duplicate_module_is_error_test() {
+    let m = Module {
+        submodules: [
+            ("main".into(), Default::default()),
+            ("main".into(), Default::default()),
+        ]
+        .into(),
+        lanes: [("main".into(), Lane::default())].into(),
+        ..Default::default()
+    };
+
+    let _ = compile(m, None).unwrap_err();
+}
+
+#[test]
+fn duplicate_lane_is_error_test() {
+    let m = Module {
+        submodules: [].into(),
+        lanes: [
+            ("main".into(), Lane::default()),
+            ("main".into(), Lane::default()),
+        ]
+        .into(),
+        ..Default::default()
+    };
+
+    let _ = compile(m, None).unwrap_err();
+}
