@@ -2,7 +2,7 @@ use crate::{vm::runtime::FieldTable, StrPointer};
 use std::convert::{From, TryFrom};
 use std::ops::{Add, Div, Mul, Sub};
 
-#[derive(Debug, Clone, Copy, PartialEq, PartialOrd)]
+#[derive(Debug, Clone, Copy, PartialOrd)]
 pub enum Value {
     Nil,
     String(StrPointer),
@@ -11,6 +11,66 @@ pub enum Value {
     Real(f64),
 }
 
+impl std::hash::Hash for Value {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        match self {
+            Value::Nil => 0u8.hash(state),
+            Value::String(s) => unsafe {
+                if let Some(s) = s.get_str() {
+                    s.hash(state);
+                }
+            },
+            Value::Integer(i) => {
+                i.hash(state);
+            }
+            Value::Real(f) => {
+                f.to_bits().hash(state);
+            }
+            Value::Object(o) => {
+                for (k, v) in unsafe { (**o).iter() } {
+                    k.hash(state);
+                    v.hash(state);
+                }
+            }
+        }
+    }
+}
+
+impl PartialEq for Value {
+    fn eq(&self, other: &Self) -> bool {
+        match (*self, *other) {
+            (Value::Nil, Value::Nil) => true,
+            (Value::String(lhs), Value::String(rhs)) => unsafe { lhs.get_str() == rhs.get_str() },
+            (Value::Object(lhs), Value::Object(rhs)) => unsafe {
+                // same objects, including null,
+                // are trivially equal
+                if lhs == rhs {
+                    return true;
+                }
+                if lhs.is_null() || rhs.is_null() {
+                    return false;
+                }
+                let lhs = &*lhs;
+                let rhs = &*rhs;
+                if lhs.len() != rhs.len() {
+                    return false;
+                }
+                for ((kl, vl), (kr, vr)) in lhs.iter().zip(rhs.iter()) {
+                    if kl != kr || vl != vr {
+                        return false;
+                    }
+                }
+                true
+            },
+            (Value::Integer(lhs), Value::Integer(rhs)) => lhs == rhs,
+            (Value::Real(lhs), Value::Real(rhs)) => lhs == rhs,
+            _ => false,
+        }
+    }
+}
+
+impl Eq for Value {}
+
 /// Intended for saving `Values` after the program has finished executing
 ///
 /// ```
@@ -76,8 +136,8 @@ impl From<Value> for OwnedValue {
                 unsafe { &*ptr }
                     .iter()
                     .map(|(k, v)| OwnedEntry {
-                        key: k.into(),
-                        value: v.into(),
+                        key: (*k).into(),
+                        value: (*v).into(),
                     })
                     .collect()
             }),
@@ -356,3 +416,21 @@ impl Div for Value {
         binary_op!(self, other, /)
     }
 }
+
+impl std::borrow::Borrow<str> for Value {
+    fn borrow(&self) -> &str {
+        match self {
+            Value::String(s) => unsafe { s.get_str().unwrap_or("") },
+            _ => "",
+        }
+    }
+}
+
+impl std::borrow::Borrow<i64> for Value {
+    fn borrow(&self) -> &i64 {
+        match self {
+            Value::Integer(i) => i,
+            _ => &(!0),
+        }
+    }
+}
